;* --------------------------------------------------------------------------------------
;*  Ted Salazar    (^_^)
;* --------------------------------------------------------------------------------------
;*  Filename:      hsi2c.asm
;*  Date:          Aug 02, 2008
;*  Version:       1.0
;*
; *****************************************************************************
; *
; * Maxim Integrated Products
; *
; * DESCRIPTION
; *
; *   Contains assembly routines customized for the MAX1239.
; *         - Main:
; *			- hsI2cRecv
; *         - fsI2cSend

; *   Allows the MAXQ2000 to bit-bang a High-Speed I2C
; *   interface at 1.7MHz (~1.64MHz). Full Speed @ 417kHz.
; *
; *   The Firmware timing works with a MAXQ2000 16MHz system clock.
; *   
; *  
; *
; *
; *****************************************************************************

; INCLUDES --------------------------------------------------------------------
$include(maxQ2000.inc)
$include(hsi2c.inc)
$include(serial_IO_1239.inc)

; CUSTOMIZATION SECTION - START -----------------------------------------------
SCL    equ 0               ; SCL pin on I/O port
SDA    equ 1               ; SDA pin on I/O port

SCL_DIR MACRO PARAM x      ; SCL direction reg bit = x
   move PD6.SCL,#x         ;
ENDM                       ;

SDA_DIR MACRO PARAM x      ; SDA direction reg bit = x
   move PD6.SDA,#x         ;
ENDM                       ;

SCL_OUT MACRO PARAM x      ; SCL output reg bit = x
   move PO6.SCL,#x         ;
ENDM                       ;

SDA_OUT MACRO PARAM x      ; SDA output reg bit = x
   move PO6.SDA,#x         ;
ENDM                       ;

SCL_IN_READ MACRO          ; C = SCL input bit
   move C,PI6.SCL          ;
ENDM                       ;

SDA_IN_READ MACRO          ; C = SDA input bit
   move C,PI6.SDA          ;
ENDM                       ;

WAIT MACRO PARAM x         ; delay time = (x+1)*50ns
   move LC[1],#x           ;
   djnz LC[1],$            ;
ENDM                       ;

; CUSTOMIZATION SECTION - END -------------------------------------------------

; EQUATES ---------------------------------------------------------------------
; MAX1239 ADDR = 0110101 (r/w bit = write) = 0x6A
I2C_ADDR7  equ 1    ; The complement of I2C addr. Complement is needed
I2C_ADDR6  equ 0    ; because the port direction is being controller,
I2C_ADDR5  equ 0    ; not the output (which is set low)
I2C_ADDR4  equ 1    ;
I2C_ADDR3  equ 0    ; i.e., 0111 111 => 1000 000
I2C_ADDR2  equ 1    ;
I2C_ADDR1  equ 0    ;
I2C_rw_bit_write  equ 1    ; r/w bit = write = logic low
I2C_rw_bit_read   equ 0    ; r/w bit = read  = logic high

; MAX1238 ADDR = 0110101 (r/w bit = write) = 0x6A
;I2C_ADDR7  equ 1    ; The complement of I2C addr. Complement is needed
;I2C_ADDR6  equ 0    ; because the port direction is being controller,
;I2C_ADDR5  equ 0    ; not the output (which is set low)
;I2C_ADDR4  equ 1    ;
;I2C_ADDR3  equ 0    ; i.e., 0111 111 => 1000 000
;I2C_ADDR2  equ 1    ;
;I2C_ADDR1  equ 0    ;
;I2C_rw_bit_write  equ 1    ; r/w bit = write = logic low
;I2C_rw_bit_read   equ 0    ; r/w bit = read  = logic high

; MAX1036 ADDR = 1100100 (r/w bit = write) = 0xC8
;I2C_ADDR7  equ 0    ; The complement of I2C addr. Complement is needed
;I2C_ADDR6  equ 0    ; because the port direction is being controller,
;I2C_ADDR5  equ 1    ; not the output (which is set low)
;I2C_ADDR4  equ 1    ;
;I2C_ADDR3  equ 0    ; i.e., 0111 111 => 1000 000
;I2C_ADDR2  equ 1    ;
;I2C_ADDR1  equ 1    ;
;I2C_rw_bit_write  equ 1    ; r/w bit = write = logic low
;I2C_rw_bit_read   equ 0    ; r/w bit = read  = logic high

; MAX1239 CONF BYTE = 01100001 =0x61 = Converts Single Ended Channel AIN0
CONF_BYTE_BIT7  equ 1    ; Complement is needed
CONF_BYTE_BIT6  equ 0    ; because the port direction is being controller,
CONF_BYTE_BIT5  equ 0    ; not the output (which is set low)
CONF_BYTE_BIT4  equ 1    ;
CONF_BYTE_BIT3  equ 1    ; i.e., 0111 111 => 1000 000
CONF_BYTE_BIT2  equ 1    ;
CONF_BYTE_BIT1  equ 1    ;
CONF_BYTE_BIT0  equ 0    ;

; MAX1239 SETUP BYTE = 11111010 =0xFA = Internal Ref, External Clock, Unipolar
SETUP_BYTE_BIT7  equ 0    ; Complement is needed
SETUP_BYTE_BIT6  equ 0    ; because the port direction is being controller,
SETUP_BYTE_BIT5  equ 0    ; not the output (which is set low)
SETUP_BYTE_BIT4  equ 0    ;
SETUP_BYTE_BIT3  equ 0    ; i.e., 0111 111 => 1000 000
SETUP_BYTE_BIT2  equ 1    ;
SETUP_BYTE_BIT1  equ 0    ;
SETUP_BYTE_BIT0  equ 1    ;
 
i2cData    equ 0    ; SRAM address to save data (up to 1024 bytes)
;nextSRAM  equ 1024 ; Next address of usable SRAM

; CODE ------------------------------------------------------------------------
org 0000h

; *****************************************************************************
; * main
; *****************************************************************************
main:
   call MAXQ_serial_port_init_8_N_1_115k
   move DPC  ,#0            ; DP[0] is in byte mode
   move A[9] ,#1            ; Init A[9]: 1 = Normal , 0 = error
   move DP[0],#i2cData      ; Addr to save bytes = i2cData
loop1: 
   call fsI2cSend           ; FULL SPEED I2C routine for configuring the MAX1239
loop2:
   move A[7] ,#2            ; ***MODIFY*** Number of bytes to read = 2
   call hsI2cRecv           ; HIGH SPEED I2C routine for reading the MAX1238
   move DP[0], #0000h       ; set pointer to location 0000h
   move LC[0],#02h           ; ***MODIFY*** Number of bytes to write = 2 
TX_Loop:
   move A[2], @DP[0]++ ;    ; A[2] = RAM Address 0x0000 & increment DP[0] after read                     
   move A[0], #PORT_0
   call serial_senddata	   
   djnz LC[0],TX_LOOP 

loop3:
;   jump main                ; Loop...
;   jump loop1               ; Loop...
;   jump loop2               ; Loop...
   jump loop3               ; Loop...

; *****************************************************************************
; fsI2cSend
;
; DESCRIPTION
;
; Inits the device for fast/standard mode I2C Send transfer as described below 
;
; fsI2cSend = WriteByte: [Addr=0x6A] [Command = CONFIGURATION BYTE = 0x61] [Data = SETUP BYTE = 0xFA]
;               		
;					[Addr=0x6A] = I2C Slave Addr 0110101 (r/w bit = write) = 0x6A
;               	[CONFIGURATION BYTE=0x61] = Converts Single Ended Channel AIN0
;					[SETUP BYTE = 0xFA] = Internal Ref, External Clock, Unipolar
;   
; PARAMS
;   A[9] = Success Status        (not preserved)
;
; REGISTER USAGE
;   A[4]  = not preserved
;   DP[0] = not preserved
;   AP    = not preserved
;
; RETURNS
;
;   A[9]  = 0 if error occured, 1 otherwise
;
; STACK USAGE
;
;   0 bytes
;
; *****************************************************************************
fsI2cSend:
   
   call hsi2cInit           ; Init I2C pins for both FULL SPEED AND HIGH SPEED

   ; --> I2C START - FULL SPEED
   SDA_DIR 1                ; FS S - drive SDA low
   WAIT 9                   ; FS S - T_HD_STA
   SCL_DIR 1                ; FS S - drive SCL low
   WAIT 19                  ; FS S - T_LOW

   move A[4],#0             ; Init A[0]=0
   move AP  ,#0             ; Acc->A[1] (I2C device address)
   move NUL ,@DP[0]--       ; Init DP[0]=DP[0]-1 because PRE-incr is used later


   ; I2C ADDR - FULL SPEED - 0110101 (r/w bit = write) = 0x6A
   SDA_DIR I2C_ADDR7        ; FS ADDR 7 - set SDA  
   nop                      ; FS ADDR 7 
   SCL_DIR 0                ; FS ADDR 7 - pull-up SCL
   WAIT 23    ;T Added                  
   SCL_DIR 1                ; FS ADDR 7 - drive SCL low
   WAIT 23                   ; FS ADDR 6
   SDA_DIR I2C_ADDR6        ; FS ADDR 6 - set SDA
   nop                      ; FS ADDR 6
   SCL_OUT 1                ; FS ADDR 6 - drive SCL high
   WAIT 9                   ; FS ADDR 6 
   SCL_OUT 0                ; FS ADDR 6 - drive SCL low
   WAIT 23                   ; FS ADDR 5 
   SDA_DIR I2C_ADDR5        ; FS ADDR 5 - set SDA
   nop                      ; FS ADDR 5 
   SCL_OUT 1                ; FS ADDR 5 - drive SCL high
   WAIT 9                   ; FS ADDR 5 
   SCL_OUT 0                ; FS ADDR 5 - drive SCL low
   WAIT 23                   ; FS ADDR 4 
   SDA_DIR I2C_ADDR4        ; FS ADDR 4 - set SDA
   nop                      ; FS ADDR 4 
   SCL_OUT 1                ; FS ADDR 4 - drive SCL high
   WAIT 9                   ; FS ADDR 4 
   SCL_OUT 0                ; FS ADDR 4 - drive SCL low
   WAIT 23                   ; FS ADDR 3 
   SDA_DIR I2C_ADDR3        ; FS ADDR 3 - set SDA
   nop                      ; FS ADDR 3 
   SCL_OUT 1                ; FS ADDR 3 - drive SCL high
   WAIT 9                   ; FS ADDR 3 
   SCL_OUT 0                ; FS ADDR 3 - drive SCL low
   WAIT 23                   ; FS ADDR 2 
   SDA_DIR I2C_ADDR2        ; FS ADDR 2 - set SDA
   nop                      ; FS ADDR 2 
   SCL_OUT 1                ; FS ADDR 2 - drive SCL high
   WAIT 9                   ; FS ADDR 2 
   SCL_OUT 0                ; FS ADDR 2 - drive SCL low
   WAIT 23                   ; FS ADDR 1 
   SDA_DIR I2C_ADDR1        ; FS ADDR 1 - set SDA
   nop                      ; FS ADDR 1 
   SCL_OUT 1                ; FS ADDR 1 - drive SCL high
   WAIT 9                   ; FS ADDR 1 
   SCL_OUT 0                ; FS ADDR 1 - drive SCL low
   WAIT 23                   ; FS ADDR 0 
   SDA_DIR I2C_rw_bit_write                ; FS ADDR 0 - set SDA low (R/W=write)
   nop                      ; FS ADDR 0 
   SCL_OUT 1                ; FS ADDR 0 - drive SCL high
   WAIT 9                   ; FS ADDR 0 
   SCL_OUT 0                ; FS ADDR 0 - drive SCL low
   WAIT 10                   ; FS ADDR 0 
   SDA_DIR 0                ; FS ADDR 0 - set SDA high     
   WAIT 10                   ; FS ADDR 0 

   ; I2C SLAVE ACK - FULL SPEED
   SCL_DIR 0                ; FS SLAVE ACK - pull-up SCL
   WAIT 3                   ; FS SLAVE ACK 
   SDA_IN_READ              ; FS SLAVE ACK - C = SDA
   jump C,hsI2cRecvEnd     ; Error 
   WAIT 2                   ; FS SLAVE ACK 
   SCL_DIR 1                ; FS SLAVE ACK - drive SCL low
   WAIT 23                  ; FS SLAVE ACK 
   
   ; I2C CONF_BYTE - FULL SPEED - =0x61 = Converts Single Ended Channel AIN0
   SDA_DIR CONF_BYTE_BIT7        ; FS ADDR 7 - set SDA  
   nop                      ; FS ADDR 7
   SCL_DIR 0                ; FS ADDR 7 - pull-up SCL
   WAIT 23    ;T Added                  
   SCL_DIR 1                ; FS ADDR 7 - drive SCL low
   WAIT 23                   ; FS ADDR 6
   SDA_DIR CONF_BYTE_BIT6        ; FS ADDR 6 - set SDA
   nop                      ; FS ADDR 6
   SCL_OUT 1                ; FS ADDR 6 - drive SCL high
   WAIT 9                   ; FS ADDR 6
   SCL_OUT 0                ; FS ADDR 6 - drive SCL low
   WAIT 23                   ; FS ADDR 5 
   SDA_DIR CONF_BYTE_BIT5        ; FS ADDR 5 - set SDA
   nop                      ; FS ADDR 5 
   SCL_OUT 1                ; FS ADDR 5 - drive SCL high
   WAIT 9                   ; FS ADDR 5
   SCL_OUT 0                ; FS ADDR 5 - drive SCL low
   WAIT 23                   ; FS ADDR 4
   SDA_DIR CONF_BYTE_BIT4        ; FS ADDR 4 - set SDA
   nop                      ; FS ADDR 4
   SCL_OUT 1                ; FS ADDR 4 - drive SCL high
   WAIT 9                   ; FS ADDR 4
   SCL_OUT 0                ; FS ADDR 4 - drive SCL low
   WAIT 23                   ; FS ADDR 3
   SDA_DIR CONF_BYTE_BIT3        ; FS ADDR 3 - set SDA
   nop                      ; FS ADDR 3
   SCL_OUT 1                ; FS ADDR 3 - drive SCL high
   WAIT 9                   ; FS ADDR 3
   SCL_OUT 0                ; FS ADDR 3 - drive SCL low
   WAIT 23                   ; FS ADDR 2 
   SDA_DIR CONF_BYTE_BIT2        ; FS ADDR 2 - set SDA
   nop                      ; FS ADDR 2 
   SCL_OUT 1                ; FS ADDR 2 - drive SCL high
   WAIT 9                   ; FS ADDR 2
   SCL_OUT 0                ; FS ADDR 2 - drive SCL low
   WAIT 23                   ; FS ADDR 1 
   SDA_DIR CONF_BYTE_BIT1        ; FS ADDR 1 - set SDA
   nop                      ; FS ADDR 1 
   SCL_OUT 1                ; FS ADDR 1 - drive SCL high
   WAIT 9                   ; FS ADDR 1 
   SCL_OUT 0                ; FS ADDR 1 - drive SCL low
   WAIT 23                   ; FS ADDR 0 
   SDA_DIR CONF_BYTE_BIT0        ; FS ADDR 1 - set SDA
   nop                      ; FS ADDR 0 
   SCL_OUT 1                ; FS ADDR 0 - drive SCL high
   WAIT 9                   ; FS ADDR 0
   SCL_OUT 0                ; FS ADDR 0 - drive SCL low  
   WAIT 20                   ; FS ADDR 0            
   SDA_DIR 0                ; FS ADDR 0 - set SDA high     
   WAIT 20                   ; FS ADDR 0

   ; I2C SLAVE ACK - FULL SPEED
   SCL_DIR 0                ; FS SLAVE ACK - pull-up SCL
   WAIT 3                   ; FS SLAVE ACK 
   SDA_IN_READ              ; FS SLAVE ACK - C = SDA
   jump C,hsI2cRecvEnd     ; Error 
   WAIT 2                   ; FS SLAVE ACK 
   SCL_DIR 1                ; FS SLAVE ACK - drive SCL low
   WAIT 23                  ; FS SLAVE ACK 


   ; I2C SETUP_BYTE - FULL SPEED - 0xFA = Internal Ref, External Clock, Unipolar
   SDA_DIR SETUP_BYTE_BIT7        ; FS ADDR 7 - set SDA  
   nop                      ; FS ADDR 7
   SCL_DIR 0                ; FS ADDR 7 - pull-up SCL
   WAIT 23    				;T Added                  
   SCL_DIR 1                ; FS ADDR 7 - drive SCL low
   WAIT 23                   ; FS ADDR 6
   SDA_DIR SETUP_BYTE_BIT6        ; FS ADDR 6 - set SDA
   nop                      ; FS ADDR 6 
   SCL_OUT 1                ; FS ADDR 6 - drive SCL high
   WAIT 9                   ; FS ADDR 6 
   SCL_OUT 0                ; FS ADDR 6 - drive SCL low
   WAIT 23                   ; FS ADDR 5 
   SDA_DIR SETUP_BYTE_BIT5        ; FS ADDR 5 - set SDA
   nop                      ; FS ADDR 5 
   SCL_OUT 1                ; FS ADDR 5 - drive SCL high
   WAIT 9                   ; FS ADDR 5 
   SCL_OUT 0                ; FS ADDR 5 - drive SCL low
   WAIT 23                   ; FS ADDR 4 
   SDA_DIR SETUP_BYTE_BIT4        ; FS ADDR 4 - set SDA
   nop                      ; FS ADDR 4 
   SCL_OUT 1                ; FS ADDR 4 - drive SCL high
   WAIT 9                   ; FS ADDR 4
   SCL_OUT 0                ; FS ADDR 4 - drive SCL low
   WAIT 23                   ; FS ADDR 3
   SDA_DIR SETUP_BYTE_BIT3        ; FS ADDR 3 - set SDA
   nop                      ; FS ADDR 3 
   SCL_OUT 1                ; FS ADDR 3 - drive SCL high
   WAIT 9                   ; FS ADDR 3 
   SCL_OUT 0                ; FS ADDR 3 - drive SCL low
   WAIT 23                   ; FS ADDR 2 
   SDA_DIR SETUP_BYTE_BIT2        ; FS ADDR 2 - set SDA
   nop                      ; FS ADDR 2 
   SCL_OUT 1                ; FS ADDR 2 - drive SCL high
   WAIT 9                   ; FS ADDR 2 
   SCL_OUT 0                ; FS ADDR 2 - drive SCL low
   WAIT 23                   ; FS ADDR 1 
   SDA_DIR SETUP_BYTE_BIT1        ; FS ADDR 1 - set SDA
   nop                      ; FS ADDR 1
   SCL_OUT 1                ; FS ADDR 1 - drive SCL high
   WAIT 9                   ; FS ADDR 1
   SCL_OUT 0                ; FS ADDR 1 - drive SCL low
   WAIT 23                   ; FS ADDR 0 
   SDA_DIR SETUP_BYTE_BIT0        ; FS ADDR 1 - set SDA
   nop                      ; FS ADDR 0 
   SCL_OUT 1                ; FS ADDR 0 - drive SCL high
   WAIT 9                   ; FS ADDR 0
   SCL_OUT 0                ; FS ADDR 0 - drive SCL low
   WAIT 20                   ; FS ADDR 0 
   SDA_DIR 0                ; FS ADDR 0 - set SDA high     
   WAIT 20                   ; FS ADDR 0 

   ; I2C SLAVE ACK - FULL SPEED
   SCL_DIR 0                ; FS SLAVE ACK - pull-up SCL
   WAIT 3                   ; FS SLAVE ACK
   SDA_IN_READ              ; FS SLAVE ACK - C = SDA
   jump C,hsI2cRecvEnd     ; Error 
   WAIT 2                   ; FS SLAVE ACK
   SCL_DIR 1                ; FS SLAVE ACK - drive SCL low
   WAIT 23                  ; FS SLAVE ACK
   move A[4],#1             ; No error


   ; I2C STOP - Full Speed
   SDA_DIR 1                ; FS STOP - drive SDA low
   SCL_DIR 1                ; FS STOP - drive SCL low
   WAIT 9                   ; FS STOP
   SCL_DIR 0                ; FS STOP - pull-up SCL
   WAIT 9                   ; FS STOP 
   SDA_DIR 0                ; FS STOP - pull-up SDA

   move A[9],A[4]           ; Return error state
   ret                      ; Return from hsi2cRecv ...


; *****************************************************************************
; hsI2cRecv
;
; DESCRIPTION
;
;   Inits the device for High-Speed I2C transfer and reads LC[0] bytes
;   from the device to SRAM starting at address DP[0]. Note that DPC must
;   be initialized such that DP[0] is in byte mode.
;
; hsI2cRecv = Frame Description: 
;				1) [FS-Start]
;               2) [FS-MASTER CODE= 0x0F + NACK] 
;               3) [HS-Repeated Start] 
;               4) [HS-MAX1239 ADDR + R/W=Read => 0x6B + ACK] 
;               5) [HS-DATA BYTE #1 (D7-D4 MSBs Dummy Highs) + (D3-D0 = B11-B8) + ACK]
;               6) [HS-DATA BYTE #2 (D7-D4 = B7-B0) + NACK]
;               7) [FS-Stop]
;               		
;
; PARAMS
;
;   A[7] = Number of bytes to read         (not preserved)
;   A[6] = SRAM address to save bytes read (not preserved)
;
; REGISTER USAGE
;
;   A[4]  = not preserved
;   DP[0] = not preserved
;   LC[1] = not preserved
;   LC[1] = not preserved
;   AP    = not preserved
;
; RETURNS
;
;   A[7]  = 0 if error occured, 1 otherwise
;
; STACK USAGE
;
;   0 bytes
;
; *****************************************************************************
hsI2cRecv:
   move DP[0],A[6]          ; Load addr to save read bytes
   move LC[0],A[7]          ; Load number of bytes to read
 
   call hsi2cInit           ; Init I2C pins

   ; --> I2C START - FULL SPEED
   SDA_DIR 1                ; FS S - drive SDA low
   WAIT 9                   ; FS S 
   SCL_DIR 1                ; FS S - drive SCL low
   WAIT 19                  ; FS S 

   move A[4],#0             ; Init A[0]=0
   move AP  ,#0             ; Acc->A[1] (I2C device address)
   move NUL ,@DP[0]--       ; Init DP[0]=DP[0]-1 because PRE-incr is used later

   ; --> I2C SEND INIT HS MASTER CODE = 0x0F + NACK - FULL SPEED
   SCL_DIR 0                ; FS ADDR 7 - pull-up SCL
   WAIT 9                   ; FS ADDR 7
   SCL_DIR 1                ; FS ADDR 7 - drive SCL low
   WAIT 23                  ; FS ADDR 7 
   SCL_DIR 0                ; FS ADDR 6 - pull-up SCL
   WAIT 9                   ; FS ADDR 6
   SCL_DIR 1                ; FS ADDR 6 - drive SCL low
   WAIT 23                  ; FS ADDR 6
   SCL_DIR 0                ; FS ADDR 5 - pull-up SCL
   WAIT 9                   ; FS ADDR 5
   SCL_DIR 1                ; FS ADDR 5 - drive SCL low
   WAIT 23                  ; FS ADDR 5 
   SCL_DIR 0                ; FS ADDR 4 - pull-up SCL
   WAIT 9                   ; FS ADDR 4 
   SCL_DIR 1                ; FS ADDR 4 - drive SCL low
   WAIT 9                   ; FS ADDR 4
   SDA_DIR 0                ; FS ADDR 3 - pull-up SDA
   WAIT 11                  ; FS ADDR 3
   SCL_DIR 0                ; FS ADDR 3 - pull-up SCL
   WAIT 9                   ; FS ADDR 3 
   SCL_DIR 1                ; FS ADDR 3 - drive SCL low
   WAIT 23                  ; FS ADDR 3 
   SCL_DIR 0                ; FS ADDR 2 - pull-up SCL
   WAIT 9                   ; FS ADDR 2 
   SCL_DIR 1                ; FS ADDR 2 - drive SCL low
   WAIT 23                  ; FS ADDR 2 
   SCL_DIR 0                ; FS ADDR 1 - pull-up SCL
   WAIT 9                   ; FS ADDR 1 
   SCL_DIR 1                ; FS ADDR 1 - drive SCL low
   WAIT 23                  ; FS ADDR 1 
   SCL_DIR 0                ; FS ADDR 0 - pull-up SCL (SDA high)
   WAIT 9                   ; FS ADDR 0 
   SCL_DIR 1                ; FS ADDR 0 - drive SCL low
   WAIT 23                  ; FS ADDR 0 

   ; I2C SLAVE NACK - FULL SPEED
   SCL_DIR 0                ; FS SLAVE NACK - pull-up SCL
   WAIT 3                   ; FS SLAVE NACK
   SDA_IN_READ              ; FS SLAVE NACK - C = SDA
   jump NC,hsI2cRecvEnd     ; Error
   WAIT 2                   ; FS SLAVE NACK 
   SCL_DIR 1                ; FS SLAVE NACK - drive SCL low
   WAIT 23                  ; FS SLAVE NACK

   ; I2C REPEATED START - HIGH SPEED
   SCL_DIR 0                ; HS Sr - pull-up SCL
   WAIT 1                   ; HS Sr
   SDA_DIR 1                ; HS Sr - drive SDA low
   WAIT 1                   ; HS Sr
   SCL_DIR 1                ; HS Sr - drive SCL low
   WAIT 1                   ; HS Sr 

   ; MAX1239 I2C ADDR - HIGH SPEED
   SDA_DIR I2C_ADDR7        ; HS ADDR 7 - set SDA  
   nop                      ; HS ADDR 7 
   SCL_DIR 0                ; HS ADDR 7 - pull-up SCL
   WAIT 2                   ;T Added                  
   SCL_DIR 1                ; HS ADDR 7 - drive SCL low
   WAIT 2                   ; HS ADDR 6
   SDA_DIR I2C_ADDR6        ; HS ADDR 6 - set SDA
   nop                      ; HS ADDR 6
   SCL_OUT 1                ; HS ADDR 6 - drive SCL high
   WAIT 1                   ; HS ADDR 6
   SCL_OUT 0                ; HS ADDR 6 - drive SCL low
   WAIT 2                   ; HS ADDR 5 
   SDA_DIR I2C_ADDR5        ; HS ADDR 5 - set SDA
   nop                      ; HS ADDR 5 
   SCL_OUT 1                ; HS ADDR 5 - drive SCL high
   WAIT 1                   ; HS ADDR 5 
   SCL_OUT 0                ; HS ADDR 5 - drive SCL low
   WAIT 2                   ; HS ADDR 4 
   SDA_DIR I2C_ADDR4        ; HS ADDR 4 - set SDA
   nop                      ; HS ADDR 4 
   SCL_OUT 1                ; HS ADDR 4 - drive SCL high
   WAIT 1                   ; HS ADDR 4 
   SCL_OUT 0                ; HS ADDR 4 - drive SCL low
   WAIT 2                   ; HS ADDR 3 
   SDA_DIR I2C_ADDR3        ; HS ADDR 3 - set SDA
   nop                      ; HS ADDR 3 
   SCL_OUT 1                ; HS ADDR 3 - drive SCL high
   WAIT 1                   ; HS ADDR 3
   SCL_OUT 0                ; HS ADDR 3 - drive SCL low
   WAIT 2                   ; HS ADDR 2 
   SDA_DIR I2C_ADDR2        ; HS ADDR 2 - set SDA
   nop                      ; HS ADDR 2 
   SCL_OUT 1                ; HS ADDR 2 - drive SCL high
   WAIT 1                   ; HS ADDR 2 
   SCL_OUT 0                ; HS ADDR 2 - drive SCL low
   WAIT 2                   ; HS ADDR 1 
   SDA_DIR I2C_ADDR1        ; HS ADDR 1 - set SDA
   nop                      ; HS ADDR 1 
   SCL_OUT 1                ; HS ADDR 1 - drive SCL high
   WAIT 1                   ; HS ADDR 1 
   SCL_OUT 0                ; HS ADDR 1 - drive SCL low
   WAIT 2                   ; HS ADDR 0 
   SDA_DIR I2C_rw_bit_read               ; HS ADDR 0 - SDA (R/W=read)
   nop                      ; HS ADDR 0 
   SCL_OUT 1                ; HS ADDR 0 - drive SCL high
   WAIT 1                   ; HS ADDR 0 
   SCL_OUT 0                ; HS ADDR 0 - drive SCL low
   WAIT 5                   ; HS ADDR 0 

   ; I2C SLAVE ACK - HIGH SPEED
   SCL_OUT 1                ; HS SLAVE ACK - drive SCL high
   nop                      ; HS SLAVE ACK 
   SDA_IN_READ              ; HS SLAVE ACK - C = SDA
   SCL_OUT 0                ; HS SLAVE ACK - drive SCL low
   jump C,hsI2cRecvEnd      ; HS SLAVE ACK - Error
   WAIT 2                   ; HS CLK STR 

hsI2cRecvData:
   ; I2C CLOCK STRETCH - HIGH SPEED
   SCL_DIR 0                ; HS CLK STR - pull-up SCL
hsI2cRecvClkStr:            ; HS CLK STR - 
   SCL_IN_READ              ; HS CLK STR - C = SCL
   jump NC, hsI2cRecvClkStr ; HS CLK STR - If SCL=0 -> wait...

   ; I2C RECV DATA (MSB) - HIGH SPEED
   SDA_IN_READ C            ; HS DATA 15 - C = SDA
   move Acc.7,C             ; HS DATA 15 - Acc.15 = C (=SDA)
   SCL_DIR 1                ; HS DATA 15 - drive SCL low
   WAIT 5                   ; HS DATA 15
   SCL_OUT 1                ; HS DATA 14 - drive SCL high
   SDA_IN_READ C            ; HS DATA 14 - C = SDA
   move Acc.6,C             ; HS DATA 14 - Acc.14 = C (=SDA)
   SCL_OUT 0                ; HS DATA 14 - drive SCL low
   WAIT 5                   ; HS DATA 14
   SCL_OUT 1                ; HS DATA 13 - drive SCL high
   SDA_IN_READ C            ; HS DATA 13 - C = SDA
   move Acc.5,C             ; HS DATA 13 - Acc.13 = C (=SDA)
   SCL_OUT 0                ; HS DATA 13 - drive SCL low
   WAIT 5                   ; HS DATA 13
   SCL_OUT 1                ; HS DATA 12 - drive SCL high
   SDA_IN_READ C            ; HS DATA 12 - C = SDA
   move Acc.4,C             ; HS DATA 12 - Acc.12 = C (=SDA)
   SCL_OUT 0                ; HS DATA 12 - drive SCL low
   WAIT 5                   ; HS DATA 12 
   SCL_OUT 1                ; HS DATA 11 - drive SCL high
   SDA_IN_READ C            ; HS DATA 11 - C = SDA
   move Acc.3,C             ; HS DATA 11 - Acc.11 = C (=SDA)
   SCL_OUT 0                ; HS DATA 11 - drive SCL low
   WAIT 5                   ; HS DATA 11 
   SCL_OUT 1                ; HS DATA 10 - drive SCL high
   SDA_IN_READ C            ; HS DATA 10 - C = SDA
   move Acc.2,C             ; HS DATA 10 - Acc.10 = C (=SDA)
   SCL_OUT 0                ; HS DATA 10 - drive SCL low
   WAIT 5                   ; HS DATA 10 
   SCL_OUT 1                ; HS DATA 09 - drive SCL high
   SDA_IN_READ C            ; HS DATA 09 - C = SDA
   move Acc.1,C             ; HS DATA 09 - Acc.9 = C (=SDA)
   SCL_OUT 0                ; HS DATA 09 - drive SCL low
   WAIT 5                   ; HS DATA 09 
   SCL_OUT 1                ; HS DATA 08 - drive SCL high
   SDA_IN_READ C            ; HS DATA 08 - C = SDA
   move Acc.0,C             ; HS DATA 08 - Acc.8 = C (=SDA)
   SCL_OUT 0                ; HS DATA 08 - drive SCL low
   WAIT 1                   ; HS DATA 08 

   move @++DP[0],Acc        ; Save byte to SRAM
   djnz LC[0],hsI2cRecvAck  ; Goto Master ACK
   jump hsi2cRecvNack       ; Goto Master NACK

hsi2cRecvAck:
   ; I2C MASTER ACK - HIGH SPEED
   SDA_DIR 1                ; HS MTR ACK - drive SDA low
   SCL_OUT 1                ; HS MTR ACK - drive SCL high
   WAIT 1                   ; HS MTR ACK
   SCL_OUT 0                ; HS MTR ACK - drive SCL low
   SDA_DIR 0                ; HS MTR ACK - pull-up SDA
   WAIT 2                   ; HS MTR ACK 

   jump hsI2cRecvData

hsi2cRecvNAck:
   ; I2C MASTER ACK - HIGH SPEED
   SCL_OUT 1                ; HS MTR NACK - drive SCL high
   WAIT 1                   ; HS MTR NACK 
   SCL_OUT 0                ; HS MTR NACK - drive SCL low
   WAIT 4                   ; HS MTR NACK 

   move A[4],#1             ; No error
hsI2cRecvEnd:

   ; I2C STOP - Full Speed
   SDA_DIR 1                ; FS STOP - drive SDA low
   SCL_DIR 1                ; FS STOP - drive SCL low
   WAIT 9                   ; FS STOP
   SCL_DIR 0                ; FS STOP - pull-up SCL
   WAIT 9                   ; FS STOP
   SDA_DIR 0                ; FS STOP - pull-up SDA

   move A[7],A[4]           ; Return error state
   ret                      ; Return from hsi2cRecv ...


; *****************************************************************************
; hsI2cSend
;
; DESCRIPTION
;
;   Inits the device for High-Speed I2C transfer and sends LC[0] bytes
;   from SRAM starting at address DP[0] to the I2C device
;
; PARAMS
;
;   LC[0] = Number of bytes to send         (destroyed)
;   DP[0] = SRAM address to save bytes send (destroyed)
;
; RETURNS
;
;   A[0]  = 0 if error occured, 1 otherwise
;
; STACK USAGE
;
;   3 bytes - AP and DPC are pushed onto the stack at the start of the
;   routine and popped off the stack at the end of the routine. The delay
;   routines use one extra stack byte for their loop counter.
;
; *****************************************************************************
hsI2cSend:
   ret                      ; No Need To Be Implemented At This Time ...   

; *****************************************************************************
; hsi2cInit - DO NOT CALL THIS ROUTINE DIRECTLY,
;             USE hsi2cRecv OR hsi2cSend
; *****************************************************************************
hsi2cInit:
   SCL_DIR 0                ; Set SCL as input
   SDA_DIR 0                ; Set SDA as input
   SCL_OUT 0                ; Set SCL output low (disable internal pull-up)
   SDA_OUT 0                ; Set SDA output low (disable internal pull-up)
   ret

; *****************************************************************************
; wait1300ns - Delay Routine
; *****************************************************************************
wait1300ns:
   push LC[0]               ;
   move LC[0],#18           ;
   DJNZ LC[0],$             ;
   pop  LC[0]               ;
   ret

; *****************************************************************************
; wait600ns - Delay Routine
; *****************************************************************************
wait600ns:
   push LC[0]               ;
   move LC[0],#4            ;
   DJNZ LC[0],$             ;
   pop  LC[0]               ;
   ret

end                         ; END!
